/* The GIMP -- an image manipulation program
 * Copyright (C) 1995-1997 Spencer Kimball and Peter Mattis
 *
 * GIMP Plug-in for Texas Instruments Pictures files.
 * Copyright (C) 2002 Romain Lievin <roms@lpg.ticalc.org>
 * Some parts from Christian Kreibich's WinIcon plugin <christian@whoop.org>.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

#include "config.h"

#include <libgimp/gimpui.h>

#define PIC_DBG

#include "main.h"
#include "picload.h"
#include "picsave.h"

#include "plugin-intl.h"

gboolean interactive_ico = FALSE;
gboolean interactive_pic = FALSE;

static void   query(void);
static void   run  (gchar      *name,
		    gint        nparams,
		    GimpParam  *param,
		    gint       *nreturn_vals,
		    GimpParam **return_vals);


GimpPlugInInfo PLUG_IN_INFO =
{
  NULL,  /* init_proc  */
  NULL,  /* quit_proc  */
  query, /* query_proc */
  run,   /* run_proc   */
};

MAIN()

static void
query(void)
{
  static GimpParamDef load_args[] =
  {
    { GIMP_PDB_INT32,    "run_mode",     "Interactive, non-interactive" },
    { GIMP_PDB_STRING,   "filename",     "The name of the file to load" },
    { GIMP_PDB_STRING,   "raw_filename", "The name entered"             }
  };
  static GimpParamDef load_return_vals[] =
  {
    { GIMP_PDB_IMAGE, "image", "Output image" },
  };
  static gint nload_args = sizeof(load_args) / sizeof(load_args[0]);
  static gint nload_return_vals = (sizeof(load_return_vals) /
				   sizeof(load_return_vals[0]));
  
  static GimpParamDef save_args[] =
  {
    { GIMP_PDB_INT32,    "run_mode",     "Interactive, non-interactive" },
    { GIMP_PDB_IMAGE,    "image",        "Input image" },
    { GIMP_PDB_DRAWABLE, "drawable",     "Drawable to save" },
    { GIMP_PDB_STRING,   "filename",     "The name of the file to save the image in" },
    { GIMP_PDB_STRING,   "raw_filename", "The name entered" },
  };
  static gint nsave_args = sizeof(save_args) / sizeof(save_args[0]);
  
  INIT_I18N();

  gimp_install_procedure("file_pic_load",
			 "Loads files of TI picture file format",
			 "Loads files of TI picture file format",
			 "Romain Lievin <roms@lpg.ticalc.org>",
			 "Romain Lievin <roms@lpg.ticalc.org>",
			 "2002",
			 N_("<Load>/TI Picture"),
			 NULL,
			 GIMP_PLUGIN,
			 nload_args, nload_return_vals,
			 load_args, load_return_vals);
  
  gimp_install_procedure("file_pic_save",
			 "Saves files in TI picture file format",
			 "Saves files in TI picture file format",
			 "Romain Lievin <roms@lpg.ticalc.org>",
			 "Romain Lievin <roms@lpg.ticalc.org>",
			 "2002",
			 N_("<Save>/TI Picture"),
			 "INDEXED, RGB",
			 GIMP_PLUGIN,
			 nsave_args, 0,
			 save_args, NULL);
  
  gimp_register_magic_load_handler("file_pic_load",
				   "73i,82i,83i,8xi,85i,86i,89i,92i,9xi,v2i",
				   "",
				    "0,string,**TI");
  gimp_register_save_handler      ("file_pic_save",
				   "73i,82i,83i,8xi,85i,86i,89i,92i,9xi,v2i",
				   ""); 
}

static void
run(gchar      *name, 
    gint        nparams, 
    GimpParam  *param, 
    gint       *nreturn_vals,
    GimpParam **return_vals)
{
  static GimpParam   values[2];
  gint32             image_ID;
  gint32             drawable_ID;
  GimpRunModeType    run_mode;
  GimpPDBStatusType  status = GIMP_PDB_SUCCESS;
  GimpExportReturnType export = GIMP_EXPORT_CANCEL;

  run_mode = param[0].data.d_int32;

  *nreturn_vals = 1;
  *return_vals  = values;
  values[0].type          = GIMP_PDB_STATUS;
  values[0].data.d_status = GIMP_PDB_EXECUTION_ERROR;

  if(strcmp(tifiles_get_version(), "0.3.9") < 0)
    {
      status = GIMP_PDB_CALLING_ERROR;
    }
  else if(strcmp(name, "file_pic_load") == 0)
    {
      switch(run_mode)
	{
        case GIMP_RUN_INTERACTIVE:
	  interactive_pic = interactive_ico = TRUE;
          break;

	case GIMP_RUN_NONINTERACTIVE:
	  interactive_pic = interactive_ico = FALSE;
	  if(nparams != 3)
	    status = GIMP_PDB_CALLING_ERROR;
	  break;

	default:
	  break;
	}

      if(status == GIMP_PDB_SUCCESS)
	{
	  image_ID = LoadPIC(param[1].data.d_string);
	  
	  if(image_ID != -1)
	    {
	      *nreturn_vals = 2;
	      values[1].type         = GIMP_PDB_IMAGE;
	      values[1].data.d_image = image_ID;
	    }
	  else
	    {
	      status = GIMP_PDB_EXECUTION_ERROR;
	     }
	}
    }
  else if(strcmp(name, "file_pic_save") == 0)
    {
      gchar *file_name;
      
      INIT_I18N_UI();

      image_ID    = param[1].data.d_int32;
      drawable_ID = param[2].data.d_int32;
      file_name   = param[3].data.d_string;

      switch(run_mode)
        {
        case GIMP_RUN_INTERACTIVE:
	  interactive_pic = interactive_ico = TRUE;
          break;

        case GIMP_RUN_NONINTERACTIVE:
	  interactive_pic = interactive_ico = FALSE;
          /*  Make sure all the arguments are there!  */
	  if(nparams < 5)
            status = GIMP_PDB_CALLING_ERROR;
          break;

        case GIMP_RUN_WITH_LAST_VALS:
	  interactive_pic = interactive_ico = FALSE;
          break;

        default:
          break;
        }

      if(status == GIMP_PDB_SUCCESS)
	{
	  status = SavePIC(file_name, image_ID);
	}

      if(export == GIMP_EXPORT_EXPORT)
	gimp_image_delete(image_ID);
    }
  else
    {
      status = GIMP_PDB_CALLING_ERROR;
    }
      
  values[0].type = GIMP_PDB_STATUS;
  values[0].data.d_status = status;
}

void
pic_cleanup(TiPicture *pic)
{
  if(!pic)
    return;

  if(pic->fp)
    fclose(pic->fp);

  tifiles_free_regular_content(pic->content);
  pic->content = NULL;
  pic->entry   = NULL;
}

gint pic_size[TIFILES_NCALCS+1][2] = 
{ 
  { 0, 0 },
  { 239, 103 },
  { 239, 103 },
  { 239, 103 },
  { 128, 63 },
  { 128, 63 },
  { 86, 63 },
  { 86, 63 },
  { 86, 63 },
  { 86, 63 },
  { 239, 103 }
};

